@@ -2254,7 +2254,7 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 					if (nmatches > 1 && swc_is_option_true(wopt_allow_ambig) == 0) {
 						/* don't allow ambiguaous selections (e.g. wildcards) unless
 						   specifically allowed via the --allow-ambig option */
-						E_DEBUG();
+						E_DEBUG("");
 						swicat_sr_form_swspec(sr, btmp2);
 						sw_e_msg(G, "SW_SELECTION_NOT_FOUND_AMBIG: %s: status=1\n",
 							strob_str(btmp2));
@@ -2279,7 +2279,7 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 					E_DEBUG2("swverify_looper_sr_payload returned %d", ret);
 					if (rp_status != 0) {
 						/* error, but continuation possible */
-						E_DEBUG();
+						E_DEBUG("");
 						if (rp_status != SWVERIFY_RP_STATUS_NO_INTEGRITY)
 							sw_e_msg(G, "SW_INTERNAL_ERROR: %s:%d status=%d\n",
 								__FILE__, __LINE__, rp_status);
@@ -2288,7 +2288,7 @@ gotoExtensionOptions:    /* Goto, Uhhg... */
 					E_DEBUG2("signal_flag=%d", G->g_signal_flag);
 					if (ret < 0) {
 						/* error that is incompatible with continuing */
-						E_DEBUG();
+						E_DEBUG("");
 						raise(SIGINT);
 						f_retval++;
 						break;
