@@ -57,47 +57,8 @@ static
 ssize_t
 try_nibble(int fd, void * buf, int timeout)
 {
-
-	return read(fd, buf, 1); /* Must block */
-
-	/* ++++ Dead code ++++
-	int flags;
-	int ret;
-	int readret;
-	time_t tm0;
-	time_t tm;
-
-	E_DEBUG("");
-	flags = fcntl(fd, F_GETFL, O_NONBLOCK);
-	ret = fcntl(fd, F_SETFL, O_NONBLOCK);
-	if (flags < 0 || ret < 0) {
-		fprintf(stderr, "fcntl: (flags=%d): %s\n", flags, strerror(errno));
-		E_DEBUG("");
-		return read(fd, buf, 1);
-	}
-
-	E_DEBUG("");
-	tm0 = time(NULL);
-	tm = tm0;
-	ret = 0;
-	while((int)(tm - tm0) < timeout && ret < 1) {
-		ret = read(fd, buf, 1);
-		if (ret < 0 && errno != EAGAIN) {
-			E_DEBUG("");
-			fprintf(stderr, "read(): %s\n", strerror(errno));
-			ret = -2;  / ** no connection ** /
-			break;
-		} else if (ret >= 0) {
-			E_DEBUG("");
-			break;
-		}
-		tm = time(NULL);
-	}
-	E_DEBUG("");
-	fcntl(fd, F_SETFL, flags);
-	E_DEBUG2("ret=%d", ret);
-	return ret;	
-	*/
+	/* must be blocking read */
+	return read(fd, buf, 1); 
 }
 
 static
@@ -314,9 +275,6 @@ write_error_line(GB * G, struct sw_logspec * logspec, int vofd, int vefd, STROB
 			E_DEBUG("");
 			news = strdup(sptr);
 			E_DEBUG2("sptr=[%s]", news);
-/* write(2, "", 0); */
-/* fprintf(stderr, "eat me: %s", news); */
-			E_DEBUG("");
 			evret = swevent_write_rpsh_event(swi_event_fd, news, len);
 			E_DEBUG("");
 			if (evret < 0) {
@@ -520,6 +478,218 @@ process_targetfile_line(STROB * tmp, char ** current_arg)
 	return strlen(line);
 }
 
+static
+void
+get_dir_component(STROB * ret, char * news, int nf)
+{
+	STROB * tmp;
+	char * s;
+	int i;
+
+	tmp = strob_open(20);
+	i = 0;
+	s = strob_strtok(tmp, news, "/");
+	while(s && i < nf) {
+		s = strob_strtok(tmp, NULL, "/");
+		i++;
+	}
+	if (s) {
+		if (strob_strtok(tmp, NULL, "/") == NULL)
+			strob_strcpy(ret, "");
+		else	
+			strob_strcpy(ret, s);
+	} else {
+		strob_strcpy(ret, "");
+	}
+	strob_close(tmp);
+}
+
+static
+STROB *
+determine_longest_common_dir(STRAR * p_sourcepath_list)
+{
+	char * s;
+	char * dirname;
+	char * dir2;
+	char * base;
+	int is_same;
+	int nf;
+	int n;
+	int nrel;
+	char * news;
+	STROB * tmp;
+	STROB * ret;
+	char * current_dir;
+	STRAR * sourcepath_list;
+
+
+	E_DEBUG("");
+	if (!p_sourcepath_list) return NULL;
+
+	sourcepath_list = strar_copy_construct(p_sourcepath_list);
+	if (!sourcepath_list) return NULL;
+
+	ret = strob_open(24);
+	tmp = strob_open(24);
+	dirname = NULL;
+
+	E_DEBUG("");
+	/* Check that the path are all relative or all
+	   absolute */
+	n = 0;
+	nrel = 0;
+	while ((s=strar_get(sourcepath_list, n))) {
+		news = strdup(s);
+		swlib_squash_all_dot_slash(news);
+		swlib_squash_double_slash(news);
+		swlib_squash_trailing_slash(news);
+		if ( *news != '/') 
+			nrel++;
+		free(news);
+		n++;
+	}
+	E_DEBUG("");
+
+	if (nrel && nrel != (n - 1)) {
+		/* commonality not supported */
+		/* mixed relative and absolute paths */
+		strob_close(ret);
+		strar_close(sourcepath_list);
+		return NULL;
+	}
+
+	/* sort the list, this algorithm requires it */
+	strar_qsort(sourcepath_list, strar_qsort_neg_strcmp);
+
+	if (nrel == 0) {
+		strob_strcpy(ret, "/");	
+	}
+
+	E_DEBUG("");
+	current_dir = NULL;
+	nf = 0;
+	is_same = 1;
+	while (is_same) {
+		E_DEBUG("");
+		nf++;
+		n = 0;
+		while ((s=strar_get(sourcepath_list, n))) {
+			E_DEBUG2("s=[%s]", s);
+			news = strdup(s);
+			swlib_squash_all_dot_slash(news);
+			swlib_squash_trailing_slash(news);
+			swlib_squash_double_slash(news);
+			get_dir_component(tmp, news, nf-1);	
+			E_DEBUG2("component is [%s]", strob_str(tmp));
+			if (current_dir == NULL) {
+				if (strob_strlen(tmp)) {
+					current_dir = strdup(strob_str(tmp));
+				} else {
+					is_same = 0;
+				}
+				E_DEBUG2("setting current_dir=[%s]", current_dir);
+			} else {
+				E_DEBUG2("current_dir is already [%s]", current_dir);
+				if (strcmp(strob_str(tmp), current_dir) != 0) {
+					/* a component is different, return the
+					   last component */
+					is_same = 0;
+					E_DEBUG3("found difference [%s] [%s]", current_dir, strob_str(tmp));
+					free(current_dir);
+					current_dir = NULL;
+					break;
+				} else {
+					;	
+				}
+			}
+			n++;
+			free(news);
+		}
+		E_DEBUG("out of while");
+		if (current_dir) {
+			E_DEBUG2("dir catting [%s]", current_dir);
+			swlib_unix_dircat(ret, current_dir);
+			free(current_dir);
+			current_dir = NULL;
+		}
+		if (s != NULL) {
+			E_DEBUG("s == NULL");
+			break;
+		}
+	}
+	E_DEBUG("");
+	strob_close(tmp);
+	E_DEBUG2("returning [%s]", strob_str(ret));
+	strar_close(sourcepath_list);
+	return ret;
+}
+
+STROB *
+swc_make_multiplepathlist(STRAR * sourcepath_listM, STROB * rootdir, STROB * firstpath)
+{
+	char * s;
+	char * b;
+	char * dir2;
+	char * base;
+	int n;
+	STROB * tmp;
+	STROB * strb;
+	STROB * lpath;
+
+	lpath = determine_longest_common_dir(sourcepath_listM);
+	if (lpath == NULL) {
+		E_DEBUG("return NULL");
+		return NULL;
+	}
+
+	/* Report the longest common directory to the caller */
+	if (rootdir) {
+		strob_strcpy(rootdir, strob_str(lpath));
+		E_DEBUG2("rootdir is [%s]", strob_str(rootdir));
+	}
+	base = strob_str(lpath);
+	E_DEBUG2("base is [%s]", base);
+
+	/* Now make a list of paths that are relative to the
+	   longest common directory */
+
+	strb = strob_open(24);
+	tmp = strob_open(24);
+	n = 0;
+
+	while ((s=strar_get(sourcepath_listM, n))) {
+		E_DEBUG2("s=[%s]", s);
+		swlib_squash_trailing_slash(s);
+		b = strstr(s, base);
+		if (b == NULL || b != s) {
+			E_DEBUG("return NULL");
+			return NULL;
+		} else {
+			dir2 = s + strlen(base);
+			if (strcmp(base, "/")) {
+				/* base is more than "/" */
+				if (*dir2 != '/') {
+					E_DEBUG("return NULL");
+					return NULL;
+				}
+				dir2++;
+			} else {
+				;	
+			}
+			E_DEBUG2("dir2=[%s]", dir2);
+			strob_sprintf(strb, 1, "\"%s\" ", (strlen(dir2) == 0 ? "." : dir2));	
+			if (firstpath)
+				strob_strcpy(firstpath, dir2);
+		}
+		E_DEBUG("");
+		n++;
+	}	
+	strob_close(tmp);
+	strob_close(lpath);
+	E_DEBUG2("list is [%s]", strob_str(strb));
+	return strb;
+}
+
 char *
 swc_print_umask(char * buf, size_t len)
 {
@@ -1225,7 +1395,8 @@ swc_process_selection_files(GB * G, VPLOB * swspec_list)
 					continue;
 				swverid = swverid_open(NULL, s);
 				if (swverid == NULL) {
-					fprintf(stderr, "%s: error processing software selection from file: [%s]\n", swlib_utilname_get(), s);
+					fprintf(stderr, "%s: error processing software selection from file: [%s]\n",
+						swlib_utilname_get(), s);
 					retval = -1;
 					goto error_out;
 				} else {
@@ -1403,7 +1574,6 @@ swc_fork_logger(GB * G, STROB * x_slinebuf, STROB * x_tlinebuf, int efd, int ofd
 	sigset_t fork_blockmask;
 	STROB * slinebuf = strob_open(100);
 	STROB * tlinebuf = strob_open(100);
-/* int xfd; */
 
 	if (s_efd) {
 		if (pipe(spipe) < 0) return -1;
@@ -1514,8 +1684,6 @@ swc_fork_logger(GB * G, STROB * x_slinebuf, STROB * x_tlinebuf, int efd, int ofd
 		done_t = 0;
 		if (!s_efd) done_s++;
 
-/* xfd = open("/tmp/log.out", O_RDWR|O_TRUNC|O_CREAT); */
-
 		while((!done_s || !done_t) && g_logger_sigterm == 0) {
 			rra = 0;
 			rrb = 0;
@@ -1534,7 +1702,6 @@ swc_fork_logger(GB * G, STROB * x_slinebuf, STROB * x_tlinebuf, int efd, int ofd
 				write_log(G, tlinebuf, logspec, ofd, efd, bufb, rrb, vlevel, ppipe[1]);
 			}
 		}
-/* close(xfd); */
 		status = 0;
 		if (g_logger_sigterm) {
 			switch(g_logger_sigterm) {
@@ -1559,15 +1726,6 @@ swc_fork_logger(GB * G, STROB * x_slinebuf, STROB * x_tlinebuf, int efd, int ofd
 			swlib_doif_writef(vlevel, SWC_VERBOSE_8, logspec, efd, "logger unexpected exit.\n");
 		}
 		swc_flush_logger(G, slinebuf, tlinebuf, efd, ofd, logspec, vlevel);
-		/*
-		close(0);
-		close(1);
-		close(2);
-		close(3);
-		close(4);
-		close(5);
-		close(6);
-		*/
 		_exit(status);
 	}
 	strob_close(slinebuf);
@@ -1710,52 +1868,8 @@ swc_set_boolean_x_option(struct extendedOptions * opta,
 		char * arg, 
 		char ** optionp)
 {
-	/*
-	if (strcasecmp(arg, "true") == 0) {
-		*optionp = "true";
-	} else {
-		*optionp = "false";
-	}
-	*/
 	set_opta(opta, nopt, arg);
 }
-		
-static
-STROB *
-make_multiplepathlist(STRAR * sourcepath_listM)
-{
-	char * s;
-	char * dirname;
-	char * dir2;
-	char * base;
-	int n;
-	STROB * tmp;
-	STROB * strb;
-
-	strb = strob_open(24);
-	tmp = strob_open(24);
-	dirname = NULL;
-	n = 0;
-
-	while ((s=strar_get(sourcepath_listM, n))) {
-		dir2 = swlib_dirname(tmp, s);
-		if (dirname == NULL) {
-			dirname = strdup(dir2);
-		}
-		if (strcmp(dir2, dirname)) {
-			/* error, not the same relative root,
-			   this condition is not supported */
-			strob_close(strb);
-			return NULL;
-		}
-		base = swlib_basename(tmp, s);
-		strob_sprintf(strb, 1, "\"%s\" ", base);	
-		n++;
-	}	
-	strob_close(tmp);
-	if (dirname) free(dirname);
-	return strb;
-}
 
 int
 swc_write_source_copy_script(GB * G,
@@ -1773,7 +1887,8 @@ swc_write_source_copy_script(GB * G,
 	int n;
 	int ret;
 	int n_source_specs;
-	char * dirname, *path;
+	char * dirname;
+	char * spath;
 	char * pax_write_command;
 	char * xx;
 	STROB * buffer, *opt1, *opt2;
@@ -1782,9 +1897,11 @@ swc_write_source_copy_script(GB * G,
 	STROB * tmp;
 	STROB * tmp1;
 	STROB * tmp2;
-	STROB * multiplepathlist;
+	STROB * mpath;
 	STROB * set_vx;
 	STROB * to_devnull;
+	STROB * rootdir;
+	STROB * firstpath;
 	int vlv;
 
 	vlv = G->g_verboseG;
@@ -1817,6 +1934,8 @@ swc_write_source_copy_script(GB * G,
 	tmp2 = strob_open(10);
 	subsh = strob_open(10);
 	subsh2 = strob_open(10);
+	rootdir = strob_open(10);
+	firstpath = strob_open(10);
 
 	if (vlv >= SWC_VERBOSE_SWIDB) {
 		strob_strcpy(set_vx, "set -vx\n");
@@ -1830,15 +1949,15 @@ swc_write_source_copy_script(GB * G,
 	dirname = strdup(strob_str(tmp));
 
 	swlib_basename(tmp, sourcepath);
-	path = strdup(strob_str(tmp));
+	spath = strdup(strob_str(tmp));
 
 	if (do_get_file_type) {
-		if (strchr(path, '\n') || 
-			strlen(path) > MAX_CONTROL_MESG_LEN - 
+		if (strchr(spath, '\n') || 
+			strlen(spath) > MAX_CONTROL_MESG_LEN - 
 				strlen(SWBIS_SWINSTALL_SOURCE_CTL_ARCHIVE ":")
 		) {
 			/*
-			 * Too long, or path Contains a newline char.
+			 * Too long, or spath Contains a newline char.
 			 */
 			return 1;
 		}
@@ -1847,26 +1966,26 @@ swc_write_source_copy_script(GB * G,
 		 */
 		strob_sprintf(opt1, 0, 
 		"echo " "\"" SWBIS_SWINSTALL_SOURCE_CTL_ARCHIVE ": %s\"",
-			path
+			spath
 			);
 		strob_sprintf(opt2, 0, 
 		"echo " "\""SWBIS_SWINSTALL_SOURCE_CTL_DIRECTORY ": %s\"",
-			 path
+			 spath
 			);
 	}
 	
 	pax_write_command = swc_get_pax_write_command(G->g_pax_write_commands, pax_write_command_key,
 						G->g_verboseG, DEFAULT_PAX_W);
 
-	if (swlib_is_sh_tainted_string(path)) { return 1; }
+	if (swlib_is_sh_tainted_string(spath)) { return 1; }
 	if (swlib_is_sh_tainted_string(dirname)) { return 1; }
 
 	if (
-		strcmp(path, "/") == 0 &&
+		strcmp(spath, "/") == 0 &&
 		strcmp(dirname, "/") == 0
 	) {
-		free(path);
-		path = strdup(".");	
+		free(spath);
+		spath = strdup(".");	
 	}
 
 	if (G->g_sourcepath_listM)
@@ -1874,22 +1993,27 @@ swc_write_source_copy_script(GB * G,
 	else
 		n_source_specs = 1;
 
-	/* fprintf(stderr, "JL n_source_specs = %d\n", n_source_specs); */
-	/* fprintf(stderr, "JL do_create = %d\n", G->g_do_createM); */
-
-	if (n_source_specs > 1)
-		multiplepathlist = make_multiplepathlist(G->g_sourcepath_listM);
-	else
-		multiplepathlist = NULL;
-
 	if (n_source_specs > 1) {
-		if (multiplepathlist == NULL)
-			n_source_specs = 1;
+		E_DEBUG("n_source_specs > 1");
+		mpath = swc_make_multiplepathlist(G->g_sourcepath_listM, rootdir, firstpath);
+		if (mpath) {
+			dirname = strdup(strob_str(rootdir));
+			spath = strdup(strob_str(firstpath));
+		}
+	} else {
+		E_DEBUG("n_source_specs not > 1");
+		mpath = NULL;
 	}
 
-	/* if (multiplepathlist) fprintf(stderr, "JL MM=%s\n", strob_str(multiplepathlist)); */
-	
-	strob_sprintf(tmp1, 0, "%s", path),
+	if (n_source_specs > 1) {
+		E_DEBUG("");
+		if (mpath == NULL) {
+			E_DEBUG("");
+			n_source_specs = 1;
+		}
+	}
+
+	strob_sprintf(tmp1, 0, "%s", spath),
 	strob_sprintf(tmp2, 0, "%s", sourcepath),
 
 	strob_strcpy(buffer_new, "");
@@ -1920,7 +2044,7 @@ swc_write_source_copy_script(GB * G,
 		"	sw_retval=$?\n"
 		"	case $sw_retval in\n"
 		"	0)\n"
-		"	if test \\( -d \"$path\" -a -r \"$path\" \\) -o \\( \"$do_create\" -a -r \"$path\" \\); then\n"
+		"	if test \"$multiple_source\" -o \\( -d \"$path\" -a -r \"$path\" \\) -o \\( \"$do_create\" -a -r \"$path\" \\); then\n"
 		"		\t\t%s\n"
 		"		\t\t%s\n"
 		"		\t\t%s\n"
@@ -1984,7 +2108,7 @@ swc_write_source_copy_script(GB * G,
 		(n_source_specs > 1) ? "x" : "",
 		blocksize,
 		dirname,
-		path,
+		spath,
 		sourcepath,
 		delaytime,
 		strob_str(opt2),
@@ -1992,7 +2116,7 @@ swc_write_source_copy_script(GB * G,
 		SEVENT(2, vlv, SW_SOURCE_ACCESS_BEGINS, ""),
 		pax_write_command,
 		pax_write_command,
-		multiplepathlist ? strob_str(multiplepathlist) : ".",
+		mpath ? strob_str(mpath) : ".",
 		SEVENT(2, vlv, SW_SOURCE_ACCESS_ENDS, "status=0"),
 		strob_str(opt1),
 		SEVENT(1, -1, SW_SOURCE_ACCESS_BEGINS, ""),
@@ -2021,10 +2145,11 @@ swc_write_source_copy_script(GB * G,
 	}
 
 	if (G->g_source_script_name) {
+		E_DEBUG2("writing source script: %s", G->g_source_script_name);
 		swlib_tee_to_file(G->g_source_script_name, -1, xx, -1, 0);
 	}
 
-	free(path);
+	free(spath);
 	free(dirname);
 	strob_close(tmp);
 	strob_close(tmp1);
@@ -2037,7 +2162,8 @@ swc_write_source_copy_script(GB * G,
 	strob_close(to_devnull);
 	strob_close(subsh);
 	strob_close(subsh2);
-	if (multiplepathlist) strob_close(multiplepathlist);
+	strob_close(rootdir);
+	if (mpath) strob_close(mpath);
 	/*
 	 * 0 is OK
 	 * !0 is error
