@@ -30,6 +30,8 @@
 #include "strar.h"
 #include "swutilname.h"
 #include "swi.h"
+#include "swicat.h"
+#include "swicat_e.h"
 #include "swicol.h"
 #include "globalblob.h"
 
@@ -67,8 +69,7 @@ int swinstall_arfinstall(GB * , SWI * swi, int ofd,
 	int * deadman, char * target_path, char * catalog_path,
 	VPLOB * swspecs, int section, int altofd, int opt_preview,
 	char * pax_read_command, int alt_catalog_root, int event_fd,
-	struct extendedOptions * eopt,
-	int keep_old_files, uintmax_t *, int allow_missing_files);
+	struct extendedOptions * eopt, int keep_old_files, uintmax_t *, int allow_missing_files, SWICAT_E * up_e);
 
 int swinstall_get_utsname_attributes(GB * , SWI * swi, int ofd, int event_fd);
 int swinstall_lib_write_trailer_blocks(int ofd, int nblocks);
