@@ -1410,9 +1410,8 @@ swpl_get_fileset_file_count(SWHEADER * infoheader)
 int
 swpl_write_out_signature_member(SWI * swi, struct tar_header * ptar_hdr,
 		struct new_cpio_header * file_hdr,
-		int ofd, int startoffset,
-		int endoffset, int signum,
-		int *package_ret)
+		int ofd, int signum,
+		int *package_ret, char * installer_sig)
 {
 	int ifd;
 	XFORMAT * xformat = swi->xformatM;
@@ -1441,6 +1440,16 @@ swpl_write_out_signature_member(SWI * swi, struct tar_header * ptar_hdr,
 	if (ret != TARRECORDSIZE) {
 		SWI_internal_fatal_error();
 	}
+	
+	/* If this is the installer sig then seek backwards since there
+	   really is not a archive member in the package for this signature */
+	 
+	if (installer_sig && strlen(installer_sig)) {
+		ret = uxfio_lseek(ifd, -TARRECORDSIZE, SEEK_CUR);
+		if (ret < 0) {
+			SWI_internal_fatal_error();
+		}
+	}
 
 	/*
 	 * Determine the filesize of the signature file
@@ -1454,6 +1463,7 @@ swpl_write_out_signature_member(SWI * swi, struct tar_header * ptar_hdr,
 	 * Do a sanity check on the size.
 	 */
 	if (filesize != 512 && filesize != 1024) {
+		fprintf(stderr, "filesize=%d\n", (int)filesize);
 		SWI_internal_fatal_error();
 	}
 
@@ -1463,17 +1473,21 @@ swpl_write_out_signature_member(SWI * swi, struct tar_header * ptar_hdr,
 	sig = malloc((size_t)filesize);
 	if (!sig) SWI_internal_fatal_error();
 
-	/*
-	 * This reads the sigfile archive member data
-	 */	
-	ret = uxfio_read(ifd, (void*)(sig), (size_t)filesize);
-	if ((int)ret != (int)filesize) {
-		SWI_internal_fatal_error();
+	if (installer_sig && strlen(installer_sig)) {
+		memset(sig, (int)'\n', (size_t)filesize);
+		memcpy(sig, installer_sig, strlen(installer_sig));
+	} else {
+		/*
+		 * This reads the sigfile archive member data
+		 */	
+		ret = uxfio_read(ifd, (void*)(sig), (size_t)filesize);
+		if ((int)ret != (int)filesize) {
+			SWI_internal_fatal_error();
+		}
 	}
-
+	
 	sig[filesize-1] = '\0';
 	siglen = strlen(sig);
-
 	/*
 	 * The package contains a sigfile padded with NULs.
 	 * We will only install the ascii bytes which means the
@@ -1586,6 +1600,18 @@ swpl_write_out_all_signatures(SWI * swi, struct tar_header * ptar_hdr,
 		SWI_internal_fatal_error();
 	}
 
+	if (swi->swi_pkgM->installer_sigM && sig_number >= 1) {
+		sig_number++;
+		ret = swpl_write_out_signature_member(swi, ptar_hdr,
+				file_hdr, ofd,
+				sig_number--, &package_ret,
+				swi->swi_pkgM->installer_sigM);
+		if (ret < 0) {
+			return ret;
+		}
+		current += ret;
+	}
+
 	/*
 	 * Loop to write out all the signatures.
 	 */
@@ -1597,8 +1623,7 @@ swpl_write_out_all_signatures(SWI * swi, struct tar_header * ptar_hdr,
 		}
 		ret = swpl_write_out_signature_member(swi, ptar_hdr,
 				file_hdr, ofd,
-				startoffset, endoffset,
-				sig_number--, &package_ret);
+				sig_number--, &package_ret, NULL);
 		if (ret < 0) {
 			return ret;
 		}
@@ -1820,6 +1845,10 @@ swpl_send_signature_files(SWI * swi, int ofd,
 			TARRECORDSIZE +  /* Trailer block */
 			TARRECORDSIZE;   /* Trailer block */
 
+	if (swi->swi_pkgM->installer_sigM && strlen(swi->swi_pkgM->installer_sigM)) {
+		stdin_file_size += (filesize + TARRECORDSIZE);
+	}
+
 	/*
 	 * FIXME    ''catalog_path'' is swi->catalog_entryM 
 	 */
@@ -3826,3 +3855,71 @@ swpl_check_package_signatures(GB * G, SWI * swi, int sig_level)
 	strob_close(gpg_status);
 	return retval;
 }
+
+char *
+swpl_make_package_signature(GB * G, SWI * swi)
+{
+	int i;
+	int retval;
+	int ret;
+	char * s;
+	char * sig;
+	int signed_bytes_fd;
+	char * signature;
+	SWGPG_VALIDATE * swgpg;
+	STROB * gpg_status;
+	int sig_block_start;
+	int sig_block_end;
+	CPLOB * archive_files;
+	SWI_FILE_MEMBER * afile;
+	char * gpg_name;
+	char * gpg_path;
+	SHCMD * sigcmd;
+	int signer_status;
+
+	retval = 0;
+	archive_files = swi->swi_pkgM->dfilesM->archive_filesM;
+
+	/*
+	 * always use agent and environment variables
+	 * get gpg_name and gpg_path from the environment
+	 * GNUPGHOME and GNUPGNAME
+	 */
+	gpg_name = getenv("GNUPGNAME");
+	gpg_path = getenv("GNUPGHOME");
+
+	E_DEBUG("");
+	if (!gpg_name || !gpg_path) {
+		/* FIXME message here */
+		if (!gpg_name)
+			fprintf(stderr, "%s: GNUPGNAME not set\n", swlib_utilname_get());
+		if (!gpg_path)
+			fprintf(stderr, "%s: GNUPGHOME not set\n", swlib_utilname_get());
+		return NULL;
+	}
+
+	E_DEBUG("");
+	sigcmd = swgpg_get_package_signature_command("GPG", gpg_name, gpg_path, SWGPG_SWP_PASS_AGENT);
+
+	E_DEBUG("");
+	swi_examine_signature_blocks(swi->swi_pkgM->dfilesM, &sig_block_start, &sig_block_end);
+	E_DEBUG("");
+	signed_bytes_fd = swlib_open_memfd();
+	E_DEBUG("");
+	swpl_write_catalog_data(swi, signed_bytes_fd, sig_block_start, sig_block_end);
+	E_DEBUG("");
+	uxfio_lseek(signed_bytes_fd, 0L, SEEK_SET);
+
+	E_DEBUG("");
+	sig = swgpg_get_package_signature(sigcmd, &signer_status,
+		SWGPG_SWP_PASS_AGENT, NULL, signed_bytes_fd, 0, NULL);
+
+	/*
+	if (sig)
+		fprintf(stderr, "%s", sig);
+	*/
+	E_DEBUG("");
+	uxfio_close(signed_bytes_fd);
+	E_DEBUG("");
+	return sig;
+}
