@@ -131,14 +131,12 @@ unsigned long int swpl_get_delivery_size(SWI * swi, int * result);
 
 int swpl_write_out_signature_member(SWI * swi, struct tar_header * ptar_hdr,
 		struct new_cpio_header * file_hdr,
-		int ofd, int startoffset,
-		int endoffset, int signum,
-		int *package_ret);
+		int ofd, int signum,
+		int *package_ret, char * installer_sig);
 
 int swpl_write_out_all_signatures(SWI * swi, struct tar_header * ptar_hdr,
 		struct new_cpio_header * file_hdr,
-		int ofd, int startoffset,
-		int endoffset, unsigned long filesize);
+		int ofd, int, int, unsigned long filesize);
 
 int swpl_write_catalog_data(SWI * swi, int ofd, int sig_block_start, int sig_block_end);
 
