@@ -25,6 +25,7 @@
 
 extern "C" {
 #include "swuser_config.h"
+#include "swextopt.h"
 }
 
 #include <stdio.h>
@@ -78,6 +79,7 @@ class swExStruct_i: public swExStruct
   	static int verboseM;		// the -x verbose=LEVEL value.
  	static int allowAbsolutePathsM;	// 
 	static int allowMissingSourceFileM; // Used when translating RPMS.
+	static struct extendedOptions * optaM; // Options structure from main()
   public:
   	swExStruct_i(char * control_dir): swExStruct() { 
 		init(); 
@@ -175,6 +177,14 @@ class swExStruct_i: public swExStruct
 	char * getDistributionLeadingPath(void) {
 			return strob_str(leadingPathM);
 	}
+ 	
+	void set_opta_array(struct extendedOptions * opta) {
+		optaM = opta;
+	}
+
+	struct extendedOptions * get_opta_array(void) {
+		return optaM; 
+	}
 
  	void setDistributionLeadingPath(char * path) {
 		strob_strcpy(leadingPathM, path);
