@@ -7,7 +7,7 @@
  * COPYING TERMS AND CONDITIONS
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * the Free Software Foundation; either version 3, or (at your option)
  * any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -17,12 +17,14 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
  */
 
+extern "C" {
 #include "swuser_config.h"
+}
+
 #include <assert.h>
-//#include "swpackage.h"
 #include "swparser.h"
 #include "swdefinitionfile.h"
 extern "C" {
@@ -45,7 +47,6 @@ extern "C" {
 }
 
 swFileMap * swDefinitionFile::swfilemapM;
-//swPtrList<swDefinition> * swDefinitionFile::swdefinition_listM;
 
 int 
 swDefinitionFile::write_fd_debug(int fd, char * prefix) {
@@ -112,7 +113,7 @@ swDefinitionFile::generateDefinitions(void) {
 
 	if (swFileMapPeek(&start_offset, &current_offset, &len) < 0) return -1;
 	if (len <= 0) {
-		fprintf (stderr,"error in generateDefinitions(): bad length.\n");
+		fprintf (stderr,"error in generateDefinitions(): bad length: %d\n", len);
 		return -2;
 	}
 	uxfio_lseek(get_mem_fd(), start_offset, SEEK_SET);
@@ -307,7 +308,7 @@ swDefinitionFile::get_next_line_offset(int swfilemapfd) {
 }
 
 char *
-swDefinitionFile::doExpandFileReference(char * attribute_value, int *len) {
+swDefinitionFile::doExpandFileReference(char * attribute_value, off_t * len) {
 	int fd;
 	STROB * contents;
 	struct stat st;
@@ -326,7 +327,7 @@ swDefinitionFile::doExpandFileReference(char * attribute_value, int *len) {
 	}
 	*len = st.st_size;
 	contents = strob_open(*len + 1);
-	if (read(fd, static_cast<void*>(strob_str(contents)), *len) != *len) {
+	if (read(fd, static_cast<void*>(strob_str(contents)), (size_t)(*len)) != *len) {
 		fprintf(stderr,"read error: %s\n", filename);
 		close(fd);
 		strob_close(contents);
