@@ -272,6 +272,10 @@ void swMetaData::init_mem(void){
     use_count_++; 
 }
 
+
+
+
+
 int
 swMetaData::add_ (char * keyword, char * value, swMetaData * newnode, int level, int off) {
    int c;
@@ -280,7 +284,11 @@ swMetaData::add_ (char * keyword, char * value, swMetaData * newnode, int level,
    newnode->set_p_offset(off);
    newnode->set_ino(off);
    c = SWPARSE_MD_TYPE_ATT;
+
+   handle_utf_verbose_top();
    ::swparse_write_attribute_att(get_mem_fd(), keyword, value, level, SWPARSE_FORM_MKUP_LEN);
+   handle_utf_verbose_bot();
+
    newnode->set_level(level);
    newnode->set_type (c);
    if (newnode != this) list_add(newnode);
@@ -305,7 +313,9 @@ char * swMetaData::set_value_ (char * val, int whence) {
 		return NULL;
 	}
 	set_p_offset(ret);
+   	handle_utf_verbose_top();
 	::swparse_write_attribute_att(memfd, ke, val, get_level(), SWPARSE_FORM_MKUP_LEN);
+   	handle_utf_verbose_bot();
     	fret = get_value((int*)NULL);
     }
     return fret;
@@ -320,6 +330,7 @@ void swMetaData::init(void)
     level_=-1;
     kw_type_=-1;
     status_= 0;
+    verbose_state_= 1;
     is_set_explicit_ = 0;
     contained_byM = NULL;
     next_ = NULL;
