@@ -75,6 +75,7 @@ class swMetaData
 	char level_;
 	char kw_type_;
 	unsigned char status_;   // 0 is included in write_fd, non-zero is excluded.
+	unsigned char verbose_state_;   // used to prevent multiple identical warning messages
 	swMetaData * contained_byM;
 
 	swMetaData * next_;
@@ -242,5 +243,22 @@ class swMetaData
 	static void setTarLinkname(char * dest, char * linkname, int size);
 	void init_mem(void);  
 	void init(void);  
+
+	void handle_utf_verbose_top(void) {
+		if (verbose_state_ == 0) {
+			swparse_set_do_not_warn_utf();
+		} else {
+			verbose_state_ = 0;
+		        swparse_unset_do_not_warn_utf();
+   		}
+	}
+
+	void handle_utf_verbose_bot(void) {
+		if (verbose_state_ == 0) {
+			swparse_set_do_not_warn_utf();
+		} else {
+			; //	
+		}
+	}
 };
 #endif
