@@ -10,7 +10,7 @@
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2, or (at your option)
+//  the Free Software Foundation; either version 3, or (at your option)
 //  any later version.
 
 //  This program is distributed in the hope that it will be useful,
@@ -20,7 +20,7 @@
 
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
 */
 
 #include <stdio.h>
@@ -65,7 +65,7 @@ public:
     virtual swStruct * get_pointer_from_index(int index)=0;
 
     virtual int write_node(int uxfio_fd)=0;
-    virtual int write(int uxfio_fd)=0;
+    virtual int iwrite(int uxfio_fd)=0;
     virtual int compare_tag(char * tag_string)=0;
     
     //
