@@ -6,7 +6,7 @@
    COPYING TERMS AND CONDITIONS:
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,7 +14,7 @@
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 
 #include "swuser_config.h"
@@ -72,11 +72,32 @@ static struct header_lookup_cache HeaderLookupCacheArray[SWMAP_MAX_LENGTH + 1];
 static int write_object_keyword(char *kw, int uxfio_ofd);
 static int write_keyword(char *kw, int uxfio_ofd);
 static int write_keyword(char *kw, int uxfio_ofd);
-static int write_req(TOPSF  * topsf, int count, int NAME_TAG, int VERSION_TAG, int FLAGS_TAG, STROB * strb, STROB * strbval, int uxfio_ofd);
+static int write__req(TOPSF  * topsf, int count, int NAME_TAG, int VERSION_TAG, int FLAGS_TAG, STROB * strb, STROB * swreq, STROB * rpmreq, char *);
 static int get_filetype(TOPSF * topsf, char * path,  mode_t mode, char ** link_s);
 static int classify_filelist(TOPSF  * topsf, char **typevector, int length);
 static int rpmHeader_get_langNum(Header h, char ** chptr);
 
+
+static
+char *
+get_sense(int flags)
+{
+    
+    if ((flags & RPMSENSE_LESS) && (flags & RPMSENSE_EQUAL))
+        return "<=";
+    else if ((flags & RPMSENSE_GREATER) && (flags & RPMSENSE_EQUAL))
+        return ">=";
+    else if (flags & RPMSENSE_LESS)
+        return "<";
+    else if (flags & RPMSENSE_GREATER)
+        return ">";
+    else if (flags & RPMSENSE_EQUAL)
+        return "==";
+    else 
+        return "==";
+
+}
+
 static
 void
 ascii_copy(STROB * dest, char * src)
@@ -92,7 +113,7 @@ ascii_copy(STROB * dest, char * src)
 		*/
 		strob_strcat(dest, " ");
 	}
-	swlib_expand_escape(&pa, &newlen, src);
+	swlib_expand_escapes(&pa, &newlen, src, NULL);
 	strob_strcat(dest, pa);
 	/* strob_strcat(dest, "\n"); */
 	free(pa);
@@ -172,6 +193,58 @@ rpmpsf_write_swdef_internal(TOPSF * topsf, int uxfio_ofd, int swdef_filetype /*
 	return ret;
 }
 
+static
+int
+rpmpsf_write_machine_name(Header h, int uxfio_ofd)
+{
+	STROB * tmp;
+	int ret;
+	char * machine_type;
+	char * os_name; 
+
+	tmp = strob_open(12);
+
+	ret = rpmpsf_get_rpmtagvalue(h, RPMTAG_ARCH, 0, (int *)NULL, tmp);
+	if (ret != 0)
+		return -1;
+
+	if (fnmatch("[ix]*86", strob_str(tmp), 0) == 0) {
+		/* its a x86 32-bit platform platform, set machine_type to
+		   "i[3-6]86 */
+		machine_type="[xi][3-6]86";
+		os_name="Linux";
+	} else if (fnmatch("x86_64", strob_str(tmp), 0) == 0) {
+		/* 64-bit */
+		machine_type="[xi]86_64";
+		os_name="Linux";
+	} else if (
+		(
+		fnmatch("BSD", strob_str(tmp), 0) == 0 ||
+		fnmatch("bsd", strob_str(tmp), 0) == 0 ||
+		0
+		) &&
+		(
+		fnmatch("[ix]*86", strob_str(tmp), 0) == 0 ||
+		0
+		) &&
+		1
+	) {
+		os_name = "*[bB][sS][dD]*";
+		machine_type = "[xi][3-6]86";
+
+	} else {
+		/* Non-Intel platforms */
+		machine_type = strob_str(tmp);
+		os_name = strob_str(tmp);
+	}
+
+	rpmpsf_write_rpm_attribute(SW_A_machine_type, machine_type, 0, 0, NULL, NULL, uxfio_ofd);
+	rpmpsf_write_rpm_attribute(SW_A_os_name, os_name, 0, 0, NULL, NULL, uxfio_ofd);
+
+	strob_close(tmp);
+	return 0;
+}
+
 int 
 rpmpsf_write_psf(TOPSF * topsf, int uxfio_ofd)
 {
@@ -564,19 +637,23 @@ rpmpsf_write_distribution(TOPSF * topsf, int uxfio_ofd, int filetype, char * fro
 	ret += rpmpsf_write_rpm_attribute("dfiles", rpmpsf_make_dfiles(topsf, dfiles, 100), RPMTAG_DISTRIBUTION, 0, h, NULL, uxfio_ofd);
 	ret += rpmpsf_write_rpm_attribute("pfiles", "pfiles", -1, 0, h, NULL, uxfio_ofd);
 
-	ret += rpmpsf_write_rpm_attribute("tag", NULL, RPMTAG_NAME, 0, h, NULL, uxfio_ofd);
+	ret += rpmpsf_write_rpm_attribute(SW_A_tag, NULL, RPMTAG_NAME, 0, h, NULL, uxfio_ofd);
 	if (topsf->smart_path_) {
+		/* Control path name prefix */
 		if (filetype != RPMPSF_FILE_PSF_SRC) {
 			if (strcmp(from_name, "."))
-			ret += rpmpsf_write_rpm_attribute("control_directory", from_name, 0, 0, h, NULL, uxfio_ofd);
+			ret += rpmpsf_write_rpm_attribute(SW_A_control_directory, from_name, 0, 0, h, NULL, uxfio_ofd);
 		} else {
-			ret += rpmpsf_write_rpm_attribute("control_directory", strob_str(topsf->control_directoryM),
+			ret += rpmpsf_write_rpm_attribute(SW_A_control_directory, strob_str(topsf->control_directoryM),
 							0, 0, h, NULL, uxfio_ofd);
 		}
-	} else
-	if (topsf->form_ == TOPSF_PSF_FORM2) {
-		ret += rpmpsf_write_rpm_attribute("control_directory", strob_str(topsf->control_directoryM),
+	} else if (topsf->form_ == TOPSF_PSF_FORM2 || topsf->form_ == TOPSF_PSF_FORM3) {
+		/* Control path name prefix */
+		ret += rpmpsf_write_rpm_attribute(SW_A_control_directory, strob_str(topsf->control_directoryM),
 							0, 0, h, NULL, uxfio_ofd);
+	} else {
+		;
+		/* no control_directory specified in the distribution object */
 	}
 
 	ret += rpmpsf_write_multilang_attribute("title",  RPMTAG_SUMMARY, h, uxfio_ofd);
@@ -633,7 +710,7 @@ rpmpsf_write_category(TOPSF * topsf, int uxfio_ofd, int filetype)
 {
 	Header h=topsf_get_rpmheader(topsf);
 	write_object_keyword("category", uxfio_ofd);
-	rpmpsf_write_rpm_attribute("tag", "rpm_group", RPMTAG_GROUP, -1, h, NULL, uxfio_ofd);
+	rpmpsf_write_rpm_attribute(SW_A_tag, "rpm_group", RPMTAG_GROUP, -1, h, NULL, uxfio_ofd);
 	return rpmpsf_write_rpm_attribute("title", (char *) (NULL), RPMTAG_GROUP, -1, h, NULL, uxfio_ofd);
 }
 
@@ -642,7 +719,7 @@ rpmpsf_write_bundle(TOPSF * topsf, int uxfio_ofd, int filetype)
 {
 	Header h=topsf_get_rpmheader(topsf);
 	write_object_keyword("bundle", uxfio_ofd);
-	return rpmpsf_write_rpm_attribute("tag", NULL, RPMTAG_GROUP, -1, h, NULL, uxfio_ofd);
+	return rpmpsf_write_rpm_attribute(SW_A_tag, NULL, RPMTAG_GROUP, -1, h, NULL, uxfio_ofd);
 }
 
 int
@@ -662,7 +739,7 @@ rpmpsf_write_vendor(TOPSF * topsf, int uxfio_ofd, int filetype)
 	}
 	write_object_keyword("vendor", uxfio_ofd);
 	ret += rpmpsf_write_rpm_attribute("the_term_vendor_is_misleading", "true", -1, 0, h, NULL, uxfio_ofd);
-	ret += rpmpsf_write_rpm_attribute("tag", name, -1, 0, h, NULL, uxfio_ofd);
+	ret += rpmpsf_write_rpm_attribute(SW_A_tag, name, -1, 0, h, NULL, uxfio_ofd);
 	{
 	switch(0) {
 		case 0:
@@ -689,7 +766,7 @@ rpmpsf_write_vendor(TOPSF * topsf, int uxfio_ofd, int filetype)
 	p1 = tagstring;
 	swlib_tr(tagstring, (int)('_'), (int)(' '));
 	ret += rpmpsf_write_rpm_attribute("the_term_vendor_is_misleading", "true", -1, 0, h, NULL, uxfio_ofd);
-	ret += rpmpsf_write_rpm_attribute("tag", tagstring, -1, 0, h, NULL, uxfio_ofd);
+	ret += rpmpsf_write_rpm_attribute(SW_A_tag, tagstring, -1, 0, h, NULL, uxfio_ofd);
 	ret += rpmpsf_write_rpm_attribute("title", (char *) NULL, RPMTAG_VENDOR, 0, h, NULL, uxfio_ofd);
 	ret += rpmpsf_write_rpm_attribute("qualifier", "seller", -1, 0, h, NULL, uxfio_ofd);
 
@@ -700,7 +777,7 @@ rpmpsf_write_vendor(TOPSF * topsf, int uxfio_ofd, int filetype)
 	}
 	write_object_keyword("vendor", uxfio_ofd);
 	ret += rpmpsf_write_rpm_attribute("the_term_vendor_is_misleading", "true", -1, 0, h, NULL, uxfio_ofd);
-	ret += rpmpsf_write_rpm_attribute("tag", (char *) NULL, RPMTAG_PACKAGER, 0, h, NULL, uxfio_ofd);
+	ret += rpmpsf_write_rpm_attribute(SW_A_tag, (char *) NULL, RPMTAG_PACKAGER, 0, h, NULL, uxfio_ofd);
 	ret += rpmpsf_write_rpm_attribute("title", (char *) NULL, RPMTAG_PACKAGER, 0, h, NULL, uxfio_ofd);
 	ret += rpmpsf_write_rpm_attribute("qualifier", "packager", 0, 0, h, NULL, uxfio_ofd);
 	strob_close(tmp);
@@ -729,8 +806,11 @@ rpmpsf_write_product(TOPSF * topsf, int uxfio_ofd, STROB * all_filesets, int fil
 	strob_strcpy(tagstring, name);
 
 	cdir = strob_str(tagstring);
-	ret += rpmpsf_write_rpm_attribute("control_directory", cdir, -1, 0, h, NULL, uxfio_ofd);
-
+	if (topsf->form_ == TOPSF_PSF_FORM3 && filetype != RPMPSF_FILE_PSF_SRC) {
+		ret += rpmpsf_write_rpm_attribute(SW_A_control_directory, "", -1, 0, h, NULL, uxfio_ofd);
+	} else {
+		ret += rpmpsf_write_rpm_attribute(SW_A_control_directory, cdir, -1, 0, h, NULL, uxfio_ofd);
+	}
 	/* Use the vendor as a base for the vendor_tag */
 	headerGetEntry(h, RPMTAG_VENDOR, &type, (void **) &name, &count);
 	if (!name) {
@@ -748,10 +828,10 @@ rpmpsf_write_product(TOPSF * topsf, int uxfio_ofd, STROB * all_filesets, int fil
 	} else {
 		strob_strcpy(rev, strob_str(tmp));
 	}
-	ret += rpmpsf_write_rpm_attribute("tag", NULL, RPMTAG_NAME, 0, h, NULL, uxfio_ofd);
-	ret += rpmpsf_write_rpm_attribute("revision", strob_str(rev), RPMTAG_VERSION, 0, h, NULL, uxfio_ofd);
-	ret += rpmpsf_write_rpm_attribute("number", NULL, RPMTAG_RELEASE, 0, h, NULL, uxfio_ofd);
-	ret += rpmpsf_write_rpm_attribute("vendor_tag", NULL, RPMTAG_RELEASE, 0, h, NULL, uxfio_ofd);
+	ret += rpmpsf_write_rpm_attribute(SW_A_tag, NULL, RPMTAG_NAME, 0, h, NULL, uxfio_ofd);
+	ret += rpmpsf_write_rpm_attribute(SW_A_revision, strob_str(rev), RPMTAG_VERSION, 0, h, NULL, uxfio_ofd);
+	ret += rpmpsf_write_rpm_attribute(SW_A_number, NULL, RPMTAG_RELEASE, 0, h, NULL, uxfio_ofd);
+	ret += rpmpsf_write_rpm_attribute(SW_A_vendor_tag, NULL, RPMTAG_RELEASE, 0, h, NULL, uxfio_ofd);
 
 	ret += rpmpsf_write_rpm_attribute("source_package", NULL, RPMTAG_SOURCE, -1, h, NULL, uxfio_ofd);
 	ret += rpmpsf_write_rpm_attribute("all_patches", NULL, RPMTAG_PATCH, -1, h, NULL, uxfio_ofd);
@@ -759,6 +839,7 @@ rpmpsf_write_product(TOPSF * topsf, int uxfio_ofd, STROB * all_filesets, int fil
 	ret += rpmpsf_write_rpm_attribute("build_host", NULL, RPMTAG_BUILDHOST, 0, h, NULL, uxfio_ofd);
 	ret += rpmpsf_write_rpm_attribute("build_root", NULL, RPMTAG_BUILDROOT, 0, h, "/", uxfio_ofd);
 	ret += rpmpsf_write_rpm_attribute("mod_time", NULL, RPMTAG_INSTALLTIME, -1, h, NULL, uxfio_ofd);
+	ret += rpmpsf_write_machine_name(h, uxfio_ofd);
 
 	if (filetype == RPMPSF_FILE_PSF_BIN) {
 		/*
@@ -823,6 +904,7 @@ rpmpsf_write_fileset(TOPSF * topsf, int uxfio_ofd, char *fileset_tag, char * fil
 	char titlestring[256];
 	char * source_prefix=topsf_get_cwd_prefix(topsf);
 	STROB *strb1;
+	char * vendor_tag;
 
 	if (!source_prefix) {
 		source_prefix=rpmpsf_make_package_prefix(topsf, (char*)NULL);
@@ -833,16 +915,27 @@ rpmpsf_write_fileset(TOPSF * topsf, int uxfio_ofd, char *fileset_tag, char * fil
 
 	if (swdef_filetype == 2 || swdef_filetype == 1 ) {
 		write_object_keyword("fileset", uxfio_ofd);
-		ret += rpmpsf_write_rpm_attribute("tag", fileset_tag, -1, 0, h, NULL, uxfio_ofd);
+		ret += rpmpsf_write_rpm_attribute(SW_A_tag, fileset_tag, -1, 0, h, NULL, uxfio_ofd);
 		if (filetype == RPMPSF_FILE_PSF_BIN)  cdir = ""; else cdir = fileset_tag; /* This line has no effect */
 		cdir = fileset_control_dir;
 		/* strob_sprintf(strb1, 0, "%s-RUN", cdir); */
-		ret += rpmpsf_write_rpm_attribute("control_directory", cdir, -1, 0, h, NULL, uxfio_ofd);
+		if (topsf->form_ == TOPSF_PSF_FORM3 && filetype != RPMPSF_FILE_PSF_SRC) {
+			ret += rpmpsf_write_rpm_attribute(SW_A_control_directory, "", -1, 0, h, NULL, uxfio_ofd);
+		} else {
+			ret += rpmpsf_write_rpm_attribute(SW_A_control_directory, cdir, -1, 0, h, NULL, uxfio_ofd);
+		}
 	}
 
 	swlib_strncpy(titlestring, fileset_tag, 200);
 	*titlestring = toupper((int) (*titlestring));
 
+	if (rpmpsf_get_rpmtagvalue(h, RPMTAG_RELEASE, 0, (int *)NULL, strb1)) {
+		SWBIS_ERROR_IMPL();
+		fprintf(stderr, "rpmpsf.c error getting RPMTAG_RELEASE\n");	
+		exit(1);
+	}
+	vendor_tag = strdup(strob_str(strb1));
+
 	if (!strcmp(fileset_tag, RPMPSF_FILESET_SOURCE)) {
 		
 		strcat(titlestring, " (pristine sources)");
@@ -887,19 +980,22 @@ rpmpsf_write_fileset(TOPSF * topsf, int uxfio_ofd, char *fileset_tag, char * fil
 
 			rpmpsf_get_rpmtagvalue(h, RPMTAG_BUILDROOT, 0, (int *) NULL, strb1);
 
+
 			ret += rpmpsf_write_rpm_attribute("excluded_arch", NULL, RPMTAG_EXCLUDEARCH, -1, h, NULL, uxfio_ofd);
 			ret += rpmpsf_write_rpm_attribute("exclude_os", NULL, RPMTAG_EXCLUDEOS, -1, h, NULL, uxfio_ofd);
 			ret += rpmpsf_write_rpm_attribute("exclusive_arch", NULL, RPMTAG_EXCLUSIVEARCH, -1, h, NULL, uxfio_ofd);
 			ret += rpmpsf_write_rpm_attribute("exclusive_os", NULL, RPMTAG_EXCLUSIVEOS, -1, h, NULL, uxfio_ofd);
 			if (ret) { SWBIS_ERROR_IMPL(); }
 
-			ret += rpmpsf_write_requisites(topsf, RPMTAG_REQUIRENAME, RPMTAG_REQUIREVERSION, RPMTAG_REQUIREFLAGS, uxfio_ofd);
+			ret += rpmpsf_write_requisites(topsf, RPMTAG_REQUIRENAME, RPMTAG_REQUIREVERSION, RPMTAG_REQUIREFLAGS,
+							uxfio_ofd, vendor_tag);
 			if (ret) { SWBIS_ERROR_IMPL(); }
 			
-			ret += rpmpsf_write_requisites(topsf, RPMTAG_CONFLICTNAME, RPMTAG_CONFLICTVERSION, RPMTAG_CONFLICTFLAGS, uxfio_ofd);
+			ret += rpmpsf_write_requisites(topsf, RPMTAG_CONFLICTNAME, RPMTAG_CONFLICTVERSION, RPMTAG_CONFLICTFLAGS,
+							uxfio_ofd, vendor_tag);
 			if (ret) { SWBIS_ERROR_IMPL(); }
 		
-			ret += rpmpsf_write_requisites(topsf, RPMTAG_PROVIDES, -1, -1, uxfio_ofd);
+			ret += rpmpsf_write_requisites(topsf, RPMTAG_PROVIDES, -1, -1, uxfio_ofd, vendor_tag);
 			if (ret) { SWBIS_ERROR_IMPL(); }
 		
 			rpmpsf_write_changelogs(topsf, uxfio_ofd);
@@ -968,7 +1064,7 @@ rpmpsf_write_product_control_file(TOPSF * topsf, int uxfio_ofd, char *tag, char
 	strcat(p, tag);
 
 	write_object_keyword("control_file", uxfio_ofd);
-	rpmpsf_write_rpm_attribute("tag", tag, -1, 0, h, NULL, uxfio_ofd);
+	rpmpsf_write_rpm_attribute(SW_A_tag, tag, -1, 0, h, NULL, uxfio_ofd);
 	rpmpsf_write_rpm_attribute("interpreter", NULL, rpm_prog, 0, h, "/bin/sh", uxfio_ofd);
 	rpmpsf_write_rpm_attribute("source", p, -1, 0, h, NULL, uxfio_ofd);
 	topsf_add_fl_entry(topsf, rpmtag, p, SWACFL_SRCCODE_RPMHEADER);
@@ -1028,7 +1124,7 @@ rpmpsf_write_rpm_attribute(char *kw, char *value, int rpmtag, int index, Header
 	}
 
 	if (kw && towrite) {
-		if (strcmp(kw, SW_A_tag) == 0) {
+		if (strcmp(kw, SW_A_tag) == 0 || strcmp(kw, SW_A_vendor_tag) == 0) {
 			/*
 			 * translate '.' to '_'
 			 */
@@ -1046,79 +1142,181 @@ rpmpsf_write_rpm_attribute(char *kw, char *value, int rpmtag, int index, Header
 }
 
 int
-rpmpsf_write_requisites(TOPSF * topsf, int NAME_TAG, int VERSION_TAG, int FLAGS_TAG, int uxfio_ofd)
+rpmpsf_write_requisites(TOPSF * topsf, int NAME_TAG, int VERSION_TAG, int FLAGS_TAG, int uxfio_ofd, char * vendor_tag)
 {
 	Header h=topsf_get_rpmheader(topsf);
 	int count, type;
 	char keyword[90], *val;
-	STROB *strb, *strbval;
+	STROB *strb;
+	STROB *swreq;
+	STROB *rpmreq;
 
 	strb = strob_open(16);
-	strbval = strob_open(16);
-	strob_strcpy(strbval, "");
+	swreq = strob_open(16);
+	rpmreq = strob_open(16);
+	strob_strcpy(swreq, "");
 
 	switch (NAME_TAG) {
 	case RPMTAG_REQUIRENAME:
 		if (!headerGetEntry(h, RPMTAG_REQUIRENAME, &type, (void **) &val, &count))
 			return 0;
 
-		if (write_req(topsf, count, NAME_TAG, VERSION_TAG, FLAGS_TAG, strb, strbval, uxfio_ofd))
+		strob_strcpy(swreq, "");
+		strob_strcpy(rpmreq, "");
+		if (write__req(topsf, count, NAME_TAG, VERSION_TAG, FLAGS_TAG, strb, swreq, rpmreq, vendor_tag))
 			return -2000;
 
-		swlib_strncpy(keyword, "prerequisites", sizeof(keyword));
+		swlib_strncpy(keyword, SW_A_prerequisites, sizeof(keyword));
 		write_keyword(keyword, uxfio_ofd);
-		swdef_write_value(strob_str(strbval), uxfio_ofd, 0, 0);
+		swdef_write_value(strob_str(swreq), uxfio_ofd, 0, 0);
+
+		swlib_strncpy(keyword, SW_A_rpm_requires, sizeof(keyword));
+		write_keyword(keyword, uxfio_ofd);
+		swdef_write_value(strob_str(rpmreq), uxfio_ofd, 0, 0);
 
 		break;
 	case RPMTAG_CONFLICTNAME:
 		if (!headerGetEntry(h, RPMTAG_CONFLICTNAME, &type, (void **) &val, &count))
 			return 0;
 
-		if (write_req(topsf, count, NAME_TAG, VERSION_TAG, FLAGS_TAG, strb, strbval, uxfio_ofd))
+		if (write__req(topsf, count, NAME_TAG, VERSION_TAG, FLAGS_TAG, strb, swreq, rpmreq, vendor_tag))
 			return -2000;
 
-		swlib_strncpy(keyword, "exrequisites", sizeof(keyword));
+		swlib_strncpy(keyword, SW_A_exrequisites, sizeof(keyword));
 		write_keyword(keyword, uxfio_ofd);
-		swdef_write_value(strob_str(strbval), uxfio_ofd, 0, 0);
+		swdef_write_value(strob_str(swreq), uxfio_ofd, 0, 0);
+
+		swlib_strncpy(keyword, SW_A_rpm_conflicts, sizeof(keyword));
+		write_keyword(keyword, uxfio_ofd);
+		swdef_write_value(strob_str(rpmreq), uxfio_ofd, 0, 0);
 
 		break;
 	case RPMTAG_PROVIDES:
 		rpmpsf_write_rpm_attribute("rpm_provides", NULL, RPMTAG_PROVIDES, -1, h, NULL, uxfio_ofd);
 		break;
 	default:
-		strob_close(strbval);
+		strob_close(swreq);
 		strob_close(strb);
+		strob_close(rpmreq);
 		return -1;
 	}
-	strob_close(strbval);
+	strob_close(swreq);
 	strob_close(strb);
+	strob_close(rpmreq);
 	return 0;
 }
 
+static
+int
+check_is_clean_tag(int flag, char * name)
+{
+	if (
+		strchr(name, ' ') ||
+		strchr(name, '/') ||
+		strchr(name, '.') ||
+		strchr(name, '(') ||
+		strchr(name, ')') ||
+		0
+	) {
+		return 1;
+	} else {
+		if (flag & RPMSENSE_RPMLIB)
+			fprintf(stderr, "rpmpsf: %s : RPMSENSE_RPMLIB is set\n", name);
+		return 0;
+	}
+}
+
 static int
-write_req(TOPSF * topsf, int count, int NAME_TAG, int VERSION_TAG,
-		int FLAGS_TAG, STROB * strb, STROB * strbval, int uxfio_ofd)
+write__req(TOPSF * topsf, int count, int NAME_TAG, int VERSION_TAG,
+		int FLAGS_TAG, STROB * tmp, STROB * swreq, STROB * rpmreq, char * vendor_tag)
 {
 	Header h=topsf_get_rpmheader(topsf);
 	int i;
-	i = 0;
-	while (i < count && !rpmpsf_get_rpmtagvalue(h, NAME_TAG, i++, (int *) NULL, strb)) {
-		strob_cat(strbval, strb);
+	int is_rpm_spec;
+	STROB * spec;
+	STROB * flags;
+	int ret;
+	char * s;
+	char * v;
+	int flag_value;
+	int res;
+	int did1 = 0;
+	int did2 = 0;
 
-		strob_strcat(strbval, ",r=");
-		if (!rpmpsf_get_rpmtagvalue(h, VERSION_TAG, i - 1, (int *) NULL, strb)) {
-			strob_strcat(strbval, strob_str(strb));
-		} else {
-			strob_strcat(strbval, "*");
+	flags = strob_open(32);
+	strob_strcpy(swreq, "");
+	strob_strcpy(rpmreq, "");
+	strob_strcpy(tmp, "");
+
+	i = 0;
+	while (i < count && !rpmpsf_get_rpmtagvalue(h, NAME_TAG, i++, (int *) NULL, tmp)) {
+
+		rpmpsf_get_rpmtagvalue(h, FLAGS_TAG, i - 1, (int *) NULL, flags);
+		flag_value = swlib_atoi(strob_str(flags), &res);
+		if (res != 0) {
+			fprintf(stderr, "error converting %s\n", strob_str(flags));
+			flag_value = 0;
 		}
-		strob_strcat(strbval, ",rpm_flag=");
-		if (!rpmpsf_get_rpmtagvalue(h, FLAGS_TAG, i - 1, (int *) NULL, strb)) {
-			strob_strcat(strbval, strob_str(strb));
+		
+		is_rpm_spec = check_is_clean_tag(flag_value, strob_str(tmp));		
+
+		if (is_rpm_spec) {
+			spec = rpmreq;
+			if (did1) strob_strcat(spec, "\n");
+			did1++;
 		} else {
-			strob_strcat(strbval, "*");
+			spec = swreq;
+			if (did2) strob_strcat(spec, "\n");
+			did2++;
 		}
-		strob_strcat(strbval, "\n");
+
+		strob_cat(spec, tmp);
+
+		rpmpsf_get_rpmtagvalue(h, VERSION_TAG, i - 1, (int *) NULL, tmp);
+		if ((s=strstr(strob_str(tmp), vendor_tag)) != NULL) {
+			if (*(s + strlen(vendor_tag)) == '\0') {
+				/* The vendor tag is part of the version
+				   chop it off */
+				*s = '\0';	
+				/* Now chop off a trailing . or - */
+				if (strob_strlen(tmp) > 0) {
+					s = strob_str(tmp) + strob_strlen(tmp) - 1;
+					if (*s == '-' || *s == '.') {
+						*s = '\0';
+					}
+				}
+				v = vendor_tag;
+			} else {
+				v = (char*)NULL;
+			}
+		} else if ((s=strchr(strob_str(tmp), '-')) != NULL) {
+			/* revision-vendor_tag */
+			if (strlen(s) > 1) {
+				*s = '\0';
+				s++;
+				v = s;
+			} else {
+				v = (char*)NULL;
+			}
+		} else {
+			v = (char*)NULL;
+		}
+
+		if (strob_strlen(tmp)) {
+			strob_sprintf(spec, 1, ",pr%s%s", get_sense(flag_value), strob_str(tmp));
+		}
+		
+		if (v != NULL && strlen(v)) {
+			strob_sprintf(spec, 1, ",v=%s", v);
+		}
+
+		if (strob_strlen(flags)) {
+			/* Implementation extension version id for RPM flags */
+			strob_sprintf(spec, 1, ",g=%s", strob_str(flags));
+		}
+
 	}
+	strob_close(flags);
 	return 0;
 }
 
@@ -1131,7 +1329,8 @@ rpmpsf_write_changelogs(TOPSF * topsf, int uxfio_ofd)
 	time_t timeval;
 	char * pa;
 	int newlen;
-	STROB *strb, *strbval;
+	STROB *strb;
+	STROB *strbval;
 
 	rpmpsf_get_rpmtagvalue(h, RPMTAG_CHANGELOGNAME, 0, &numchanges, NULL);
 	if (!numchanges)
@@ -1164,7 +1363,7 @@ rpmpsf_write_changelogs(TOPSF * topsf, int uxfio_ofd)
 			return -1;
 		}
 		strob_cat(strbval, strb);
-		strob_strcat(strbval, "\n");
+		/* strob_strcat(strbval, "\n"); */
 		rpmpsf_write_rpm_attribute("change_log", strob_str(strbval), -1, -1, h, NULL, uxfio_ofd);
 	}
 	strob_close(strb);
