@@ -8,7 +8,7 @@
  * COPYING TERMS AND CONDITIONS
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * the Free Software Foundation; either version 3, or (at your option)
  * any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,12 +18,13 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
  */
 
 #define FILENEEDDEBUG 1 /* controls the E_DEBUG macros */
 #undef FILENEEDDEBUG
 
+#include "swuser_config.h"
 #include <ctype.h>
 #include <sys/types.h>
 #include <pwd.h>
@@ -112,7 +113,7 @@ internal_redirect(
 			if ((fd=open(srcfile, O_RDONLY, 0)) == -1) {
 				fprintf (stderr,
 					"fatal: error opening %s\n", srcfile);
-				_exit(0);
+				_exit(1);
 			}
 			if (fd != STDIN_FILENO) {
 				dup2(fd, STDIN_FILENO);
@@ -120,7 +121,7 @@ internal_redirect(
 		} else {
 			fprintf (stderr,
 			"fatal: internal error, invalid stdin redirection. \n");
-			_exit(0);
+			_exit(1);
 		}
 	}
 
@@ -143,7 +144,7 @@ internal_redirect(
 			if ((fd=open(dstfile, flags, 0666)) == -1) {
 				fprintf (stderr,
 					"fatal: error opening %s\n", dstfile);
-				_exit(0);
+				_exit(1);
 			}
 			umask(mode);
 			if (fd != STDOUT_FILENO) {
@@ -160,7 +161,7 @@ internal_redirect(
 			fprintf (stderr,
 				"fatal: internal error,"
 				" invalid stdout redirection. \n");
-			_exit(0);
+			_exit(1);
 		}
 	}
 
@@ -183,7 +184,7 @@ internal_redirect(
 			if ((fd = open(errfile, flags, 0666)) == -1) {
 				fprintf (stderr,
 					"fatal: error opening %s\n", errfile);
-				_exit(0);
+				_exit(1);
 			}
 			umask(mode);
 			if (fd != STDERR_FILENO) {
@@ -195,7 +196,7 @@ internal_redirect(
 		} else {
 			fprintf (stderr,"fatal: internal error,"
 					" invalid stderr redirection. \n");
-			_exit(0);
+			_exit(1);
 		}
 	}
 }
@@ -206,8 +207,11 @@ shcmd_common_open(char * cmdstring)
 {
 	mode_t mode = (mode_t)0;
 	int i;
-	SHCMD * shcmd=(SHCMD*)malloc(sizeof(SHCMD));
-        
+	SHCMD * shcmd;
+
+	shcmd=(SHCMD*)malloc(sizeof(SHCMD));
+       	if (shcmd == NULL) return NULL;
+ 
 	shcmd->argvector_ = cplob_open(10);
 	shcmd->argv_set_=0;
 
@@ -236,6 +240,8 @@ shcmd_common_open(char * cmdstring)
 	shcmd->umask_ = mode;
 	(shcmd->user_)[0] = '\0';  /* empty string */
 	(shcmd->group_)[0] = '\0';  /* empty string */
+	shcmd->f_filter_ =  (int (*) (int, int, void*))(NULL);
+	shcmd->f_filter_ctrl_ = (void*)NULL;
 	shcmd->f_exec_ = shcmd_unix_execve;
 	for (i=0; i < CLOSE_LIST_LEN; i++) {
 		shcmd->close_list_[i] = -1;
@@ -249,13 +255,14 @@ close_list(SHCMD * shcmd)
 {
 	int fd;
 	int i = 0;
-	fd = shcmd->close_list_[i];
-	while (fd >= 0 && i < CLOSE_LIST_LEN) {
+	while (i < CLOSE_LIST_LEN) {
 		/* fprintf(stderr, "shcmd : closing [%d]\n", fd); */
-		if (close(fd) < 0) 
-			fprintf(stderr,"shcmd : close_list_: fd=%d : %s\n",
+		fd = shcmd->close_list_[i];
+		if (fd >= 0)
+			if (close(fd) < 0) 
+				fprintf(stderr,"shcmd : close_list_: fd=%d : %s\n",
 							fd, strerror(errno));
-		fd = shcmd->close_list_[++i];
+		i++;
 	}
 }
 
@@ -283,7 +290,7 @@ cmd_invoke(SHCMD *cmd)
 		if (shcmd_getgid(cmd) >= 0) {
 			if (setgid((gid_t)(shcmd_getgid(cmd)))) {
 				fprintf(stderr, "setgid failed in"
-					" shcmd_cmd_invoke, group=%s %s", 
+					" shcmd_cmd_invoke, group=%s %s\n", 
 						cmd->group_, strerror(errno));
 			}
 		} else if (gid == -2) {
@@ -301,7 +308,7 @@ cmd_invoke(SHCMD *cmd)
 		if (uid >= 0) {
 			if(setuid((uid_t)(uid))) {
 				fprintf(stderr, "setuid failed in"
-					" shcmd_cmd_invoke, user=%s %s", 
+					" shcmd_cmd_invoke, user=%s %s\n", 
 						cmd->user_, strerror(errno));
 			}
 		} else if (uid == -2) {
@@ -327,13 +334,21 @@ cmd_invoke(SHCMD *cmd)
 					cmd->errfile_, 
 					cmd->append2_);
 		umask(cmd->umask_);
-		if (cmd->argv_ && cmd->argv_[0] && strlen(cmd->argv_[0])) {
+		if (cmd->argv_ && cmd->argv_[0] && strcmp(cmd->argv_[0], SHCMD_INTERNAL_FILTER) == 0) {
+			int ret;
+			if (cmd->f_filter_) {
+				ret = (*(cmd->f_filter_))(STDOUT_FILENO, STDIN_FILENO, cmd->f_filter_ctrl_);
+			} else {		
+				ret = swlib_pipe_pump(STDOUT_FILENO, STDIN_FILENO);
+			}
+			child_exitval = ret < 0 ? 1 : 0;
+		} else if (cmd->argv_ && cmd->argv_[0] && strlen(cmd->argv_[0])) {
  			(*(cmd->f_exec_))(cmd);
 			fprintf (stderr,"%s: unable to execute %s\n", swlib_utilname_get(), cmd->argv_[0]);	
 			child_exitval = 126;
 		} else {
-			swlib_pipe_pump(STDOUT_FILENO, STDIN_FILENO);
-			child_exitval = 0;
+			fprintf(stderr,	"shcmd.c: illegal or null command name\n");
+			child_exitval = 1;
 		}
 		_exit(child_exitval);
 	} else {
@@ -402,11 +417,19 @@ char *
 shcmd_find_in_path(char * ppath, char * pgm)
 {
 	char * retval = NULL;
-	STROB * tmp;	
+	struct stat st;
+	STROB * tmp;
 	STROB * name;
 	char * s;
+	int ret;
+
+	if (strchr(pgm, '/')) {
+		ret = stat(pgm, &st);
+		if (ret < 0) return NULL;
+		if (!S_ISREG(st.st_mode)) return NULL;
+		return pgm;
+	}
 
-	if (strchr(pgm, '/')) return pgm;
 	if (!ppath) return (char*)NULL;
 	tmp = strob_open(10);
 	name = strob_open(10);
@@ -452,8 +475,10 @@ shcmd_reap_child(SHCMD * cmd, int flag)
 		return 0;
 	}
 
+	E_DEBUG("");
 	pid = waitpid(cmd->pid_, &(cmd->status_), flag);
 	if (pid < 0) {
+		E_DEBUG("ERROR");
 		if (errno == ECHILD) {
 			E_DEBUG("ECHILD");
 			cmd->child_gone_ = 1;
@@ -463,6 +488,7 @@ shcmd_reap_child(SHCMD * cmd, int flag)
 			(int)(cmd->pid_), cmd->argv_[0], strerror(errno));
 	} else if (pid > 0) {
 		E_DEBUG3("Normal stop: pid=%d: %s", (int)(cmd->pid_), cmd->argv_[0]);
+		E_DEBUG3("exit value for %s is %d", cmd->argv_[0], shcmd_get_exitval(cmd));
 		cmd->child_gone_ = 1;
 	}
 	return pid;
@@ -570,19 +596,24 @@ shcmd_wait(SHCMD ** cmd_vec)
 	pid_t pid;
 	int ncmds = 0;
 	int i;
+	int ret;
 	
 	while(*cmdv) {
 		cmdv++;
 		ncmds++;
 	}
 	cmdv--;	
+	E_DEBUG("");
 	if (shcmd_reap_child(*cmdv, 0) > 0) {
+		E_DEBUG("");
 		ncmds--;	
 		for(i=0; i<ncmds; i++) {
+			E_DEBUG("");
 			pid=shcmd_reap_child(cmd_vec[i], 0);
 		}
 	}
-	return shcmd_get_exitval(*cmdv);
+	ret = shcmd_get_exitval(*cmdv);
+	return ret;
 }
 
 int
