@@ -8,7 +8,7 @@
  * COPYING TERMS AND CONDITIONS
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * the Free Software Foundation; either version 3, or (at your option)
  * any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
  */
 
 #include "swuser_config.h"
@@ -95,7 +95,7 @@ swdef_write_value_to_buffer(STROB * ob, char *value)
 	if ((s = strstr(value, "\n"))) {
 		*s = '\0';
 	}
-	swlib_expand_escape(&newbuf, &newlen, value);
+	swlib_expand_escapes(&newbuf, &newlen, value, (STROB*)NULL);
 	if (newlen == 0) {
 		strob_strcat(ob, "\"\"");
 		ret = 2;
@@ -124,7 +124,8 @@ swdef_write_value(char *value, int uxfio_ofd,
 				int value_length, int quote_off)
 {
 	char buf0[8];
-	char *newbuf = NULL, *s;
+	char * newbuf = NULL;
+	char * s;
 	int newlen, ret = 0;
 
 
@@ -139,13 +140,18 @@ swdef_write_value(char *value, int uxfio_ofd,
 				*s = '\0';
 			}
 		}
-		swlib_expand_escape(&newbuf, &newlen, value);
+		swlib_expand_escapes(&newbuf, &newlen, value, (STROB*)NULL);
 		if (!newlen) {
 			sprintf(buf0, "\"\"");
 			ret = uxfio_write(uxfio_ofd, buf0, strlen(buf0));
-		} else if (strstr(value, "\\n") || 
-					strstr(value, "\n") || 
-						strpbrk(value, "#\"\\")) {
+		} else if (
+				strstr(value, "\\n") || 
+				strstr(value, "\n") || 
+				((s=strstr(value, "<")) && s != value) || 
+				strstr(value, ">") || 
+				strpbrk(value, "#\"\\") ||
+				0
+		) {
 			sprintf(buf0, "\"");
 			if (quote_off == 0) 
 				ret = uxfio_write(uxfio_ofd, buf0, 
@@ -163,69 +169,3 @@ swdef_write_value(char *value, int uxfio_ofd,
 		swbis_free(newbuf);
 	return ret;
 }
-
-int
-swlib_expand_escape(char **pa, int *newlen, char *src)
-{
-	int n = strlen(src);
-	int count = 0;
-	int i = 0, j = 0, k = 0;
-	char *lp = src;
-	STROB *store;
-
-	store = strob_open(20);
-	if (!store)
-		return -1;
-	while (k < n) {
-		if (*(lp + k) == '\\') {
-			count++;
-			if (count % 2 && (*(lp + k + 1) == 'n')) {
-				count=0;
-				strob_chr_index(store, i + j, '\n');
-				 k++;
-			} else {
-				strob_chr_index(store, i + j, *(lp + k));
-				if (count % 2) {
-					if (
-					!swlib_is_escapable(*(lp + k + 1))
-					) {
-						j++;
-						strob_chr_index(store,
-								i + j,
-								*(lp + k));
-					} else {
-						;
-					}
-				}
-			}
-		} else if ((*(lp+k) < (int)7 && *(lp+k) > 0) || *((unsigned char*)(lp+k)) >= 127) {
-			strob_sprintf(store, 1, "\\x%02X", *((unsigned char*)(lp+k)));
-			j+=3;
-		} else {
-			count = 0;
-			if (*(lp + k) == '#') {
-				if (strob_get_char(store,  i + j - 1) != '\\') {
-					strob_chr_index(store, i + j, '\\');
-					j++;
-				}	
-				strob_chr_index(store,i + j, '#');
-			
-			} else if (*(lp + k) == '\"') {
-				if (strob_get_char(store, i + j - 1) != '\\') {
-					strob_chr_index(store, i + j, '\\');
-					j++;
-			
-				}
-				strob_chr_index(store, i + j, '\"');
-			} else {
-				strob_chr_index(store, i + j, *(lp + k));
-			}
-		}
-		i++;
-		k++;
-	}
-	strob_chr_index(store, i + j, '\0');
-	*pa = strob_release(store);
-	*newlen = i + j;
-	return 0;
-}
