@@ -90,7 +90,7 @@ synct_i_read(SYNCT * synct, int fd, char * buf)
 
 static
 char * 
-synct_look_for_eof(SYNCT * synct, char ** p_trailer_start)
+synct_look_for_eof(SYNCT * synct, unsigned char ** p_trailer_start)
 {
 	static int d;
 	unsigned char * ibuf;
@@ -1584,7 +1584,7 @@ swlib_synct_read(SYNCT * synct, int fd, void * buf)
 	unsigned char * ibuf;
 	unsigned char * tbuf;
 	char * pos;
-	char * provisional_trailer_start;
+	unsigned char * provisional_trailer_start;
 	int ret;
 	int tmpret;
 	int rereads;
