@@ -8,7 +8,7 @@
    COPYING TERMS AND CONDITIONS
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * the Free Software Foundation; either version 3, or (at your option)
  * any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
  */
 
 
@@ -429,6 +429,22 @@ swsdflt_get_has_default_value(struct swsdflt_defaults * swd){
 
 o__inline__
 char * 
-swsdflt_get_default_value(struct swsdflt_defaults * swd){
+swsdflt_get_default_value_by_ent(struct swsdflt_defaults * swd){
 	return swd->value;
 }
+
+char *
+swsdflt_get_default_value(char * object, char * keyword)
+{
+   struct swsdflt_defaults *ent;
+   char * s;
+   if ((s=strchr(object, '.')) != NULL) {
+      *s = '\0';
+      ent = swsdflt_return_entry(object, s+1); 
+      *s = '.';
+   } else {
+      ent = swsdflt_return_entry(object, keyword); 
+   }
+   if (!ent) return NULL;
+   return ent->value;	
+}
