@@ -7,7 +7,7 @@
  * COPYING TERMS AND CONDITIONS
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
+ * the Free Software Foundation; either version 3, or (at your option)
  * any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  
  */
 
 
@@ -43,7 +43,7 @@
 struct fileChain {
 	char *nameFC;
 	int header_offsetFC;
-	int data_offsetFC; 
+	intmax_t data_offsetFC; 
 	struct fileChain * prevFC;
 	struct fileChain * nextFC;
 	int uxfio_u_fdFC;
@@ -105,8 +105,8 @@ typedef struct {
 	
 	/* General */	
 	int uxfio_buftype;
-	int current_header_offsetM;
-	int current_data_offsetM;
+	intmax_t current_header_offsetM;
+	intmax_t current_data_offsetM;
 	int is_unix_pipeM;
 	int eoaM;			/* End of archive reached ?? */
 	int makefilechainM;		/* Override in-order traversal optimization. */
@@ -140,6 +140,7 @@ char * 		swvarfs_u_get_linkname	(SWVARFS * swvarfs, int fd);
 char * 		swvarfs_u_get_name	(SWVARFS * swvarfs, int fd);
 void 		swvarfs_stop_function	(SWVARFS * swvarfs, int (*fc)(void*, char *, char *));
 char * 		swvarfs_get_next_dirent (SWVARFS * swvarfs, struct stat *st);
+int 		swvarfs_dirent_reset	(SWVARFS * swvarfs);
 int 		swvarfs_dirent_err	(SWVARFS * swvarfs);
 int		swvarfs_get_format	(SWVARFS * swvarfs);
 int 		swvarfs_file_has_data	(SWVARFS * swvarfs);
