@@ -6,7 +6,7 @@
    COPYING TERMS AND CONDITIONS:
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,7 +14,7 @@
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #include "swuser_config.h"
 #include <stdio.h>
@@ -95,8 +95,8 @@ make_case_pattern_for_unused_tags(STROB * buf)
 	}
 	strob_sprintf(buf, STROB_DO_APPEND, ")\n");
 	strob_sprintf(buf, STROB_DO_APPEND, 
-		"\t\t\techo \"$0: no script for this tag\" 1>&2\n"
-		"\t\t\texit 0\n"
+		"\t\t\techo \"$0: no ${SWBCS_SCRIPT_TAG} script for ${SWBCS_MATCH}\" 1>&2\n"
+		"\t\t\texit " SWBIS_STATUS_COMMAND_NOT_FOUND "\n"
 		"\t\t\t;;\n"
 		);
 }
