@@ -5,7 +5,7 @@
    COPYING TERMS AND CONDITIONS:
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,7 +13,7 @@
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #ifndef swi_common_h_200501
 #define swi_common_h_200501
@@ -84,5 +84,6 @@ void swi_com_header_manifold_reset(SWHEADER * swheader);
 int swi_com_close_memfd(int fd);
 int swi_com_set_header_index(SWHEADER * header, SWPATH_EX *, int * ai);
 /* char * swi_com_determine_control_directory(SWHEADER * header); */
+char * swi_com_new_fd_mem(int fd, int * datalen);
 
 #endif
