@@ -14,7 +14,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -24,7 +24,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 
 #include "swuser_config.h"
@@ -482,12 +482,13 @@ taru_set_filetype_from_tartype(char ch, mode_t * mode, char * filename)
 				(*mode) |= CP_IFREG;
 		}
 		break;
+	case NOTDUMPEDTYPE:
+		(*mode) |= CP_IFSOCK;
+		break;
 	default:
 		(*mode) |= CP_IFREG;
 		fprintf(stderr, 
-		"taru_set_filetype_from_tartype():"
-		" warning:  typeflag [%c] not supported, set to Reg. File.\n",
-				ch);
+		"%s: warning:  typeflag [%c] not supported, ignoring file\n", swlib_utilname_get(), ch);
 		return 1;
 	}
 	return 0;
@@ -499,6 +500,9 @@ tarui_get_filetypes (mode_t mode, int * cpio_mode, char *tarflag)
 	if (S_ISREG(mode)) {
 		*tarflag=REGTYPE;
 		*cpio_mode= CP_IFREG;
+	} else if (S_ISDOOR(mode)) {
+		*tarflag=NOTDUMPEDTYPE;
+		*cpio_mode= CP_IFSOCK;
 	} else if (S_ISDIR(mode)) {
 		*tarflag=DIRTYPE;
 		*cpio_mode = CP_IFDIR;
@@ -524,7 +528,7 @@ tarui_get_filetypes (mode_t mode, int * cpio_mode, char *tarflag)
 #endif
 #ifdef S_ISSOCK
 	} else if (S_ISSOCK(mode)) {
-		*tarflag=FIFOTYPE;
+		*tarflag=NOTDUMPEDTYPE;
 		*cpio_mode= CP_IFSOCK;
 #endif
 #ifdef S_ISNWK
@@ -533,7 +537,8 @@ tarui_get_filetypes (mode_t mode, int * cpio_mode, char *tarflag)
 		*cpio_mode= CP_IFNWK;
 #endif
 	} else {
-		fprintf (stderr,"internal error in taru_get_cpio_filetype.\n");	
+		fprintf (stderr,"%s: unrecognized type in mode: %d\n", swlib_utilname_get(), (int)mode);	
+		*tarflag=(char)(-1);
 		*cpio_mode= -1;
 	}
 }
@@ -561,7 +566,7 @@ taru_get_cpio_filetype (mode_t mode)
 int
 taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 			int in_des, char * fheader_buffer,
-			int * eoa, int tarheaderflags)
+			int * eoa, int tarheaderflags, int buf_len)
 {
 	mode_t modet;
 	int retval = 0;
@@ -576,11 +581,14 @@ taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 	char * header_buffer = fheader_buffer;
 	int do_record_header = 0;
 	int tar_iflags_retain_header_id = 0;
+	int buffer_offset;
+	int tmpret;
 #ifndef __MSDOS__
 	long uidp;
 	long gidp;
 #endif
 
+	buffer_offset = 0;
 	if (taru) {
 		tar_iflags_retain_header_id = (tarheaderflags & TARU_TAR_RETAIN_HEADER_IDS);
 	}
@@ -598,15 +606,21 @@ taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 	
 	GNU_L_REREAD:	
 	
-	if (!header_buffer) {
-		if (taru_tape_buffered_read(in_des, (void *)(&tar_rec),
-					TARRECORDSIZE) != TARRECORDSIZE) {
-			fprintf(stderr,
-				"tar.c: short read in read_in_tar_header2\n");
+	if (header_buffer == NULL) {
+		tmpret = taru_tape_buffered_read(in_des, (void *)(&tar_rec), TARRECORDSIZE);
+		if (tmpret > 0 && tmpret != TARRECORDSIZE) {
+			fprintf(stderr, "otar.c: short read in read_in_tar_header2 return=%d\n", tmpret);
 			return -1;
+		} else if (tmpret == 0) {
+			return 0;
+		} else if (tmpret < 0) {
+			fprintf(stderr, "otar.c: read error from read_in_tar_header2 return=%d\n", tmpret);
+			return -1;
+		} else {
+			; /* Ok */
 		}
 	} else {
-		memcpy((void *) (&tar_rec), header_buffer, TARRECORDSIZE);
+		memcpy((void *) (&tar_rec), header_buffer+buffer_offset, TARRECORDSIZE);
 	}
 
 	if (do_record_header) {
@@ -646,22 +660,58 @@ taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 	}
 
 	if (*(((char*)tar_hdr) + 156) == 'L') {
-		if (header_buffer) {
+		if (header_buffer && buf_len == TARRECORDSIZE) {
 			/*
 			* your screwed.
 			*/
 			fprintf(stderr,
 			"fatal error, can not read GNU type L extension\n");
 			exit(24);
+		} else if (header_buffer && buf_len > TARRECORDSIZE) {
+			taru_otoumax(tar_hdr->size, &file_hdr->c_filesize);
+			if (file_hdr->c_filesize > 511) {
+				/*
+				* pretend can't be done. At the moment
+				* filenames this long are not supported.
+				*/
+				fprintf(stderr,
+	"fatal error, can not read GNU type L extension because file name\n"
+	"is too long.\n");
+				exit(24);
+			}
+			long_link_bytes += 512;
+			retval += 512;
+
+			/* Sanity check */
+			if (retval + long_link_bytes != buf_len) {
+				fprintf(stderr, "fatal error in taru_in_tar_header2: type L error\n");
+				exit(24);
+			}
+			long_link_buffer = malloc(TARRECORDSIZE + 1);
+			memset(long_link_buffer, '\0', TARRECORDSIZE + 1);
+			memcpy(long_link_buffer, header_buffer+512, 512);
+			long_link_buffer[TARRECORDSIZE] = '\0';
+
+			if (do_record_header) {
+				strob_set_memlength(taru->headerM,
+						taru->header_lengthM + 512);
+				memcpy((void*)(strob_str(taru->headerM) +
+							 taru->header_lengthM),
+					(void*)(long_link_buffer), 512);
+				taru->header_lengthM += 512;
+			}
+			buffer_offset = TARRECORDSIZE + TARRECORDSIZE;
+			goto GNU_L_REREAD;
 		} else {
 			/* 
 			* read the data block which contains the filename.
 			*/
-			taru_otoul(tar_hdr->size, &file_hdr->c_filesize);
+			taru_otoumax(tar_hdr->size, &file_hdr->c_filesize);
 
 			if (file_hdr->c_filesize > 511) {
 				/*
-				* pretend can't be done.
+				* pretend can't be done. At the moment
+				* filenames this long are not supported.
 				*/
 				fprintf(stderr,
 	"fatal error, can not read GNU type L extension because file name\n"
@@ -670,6 +720,7 @@ taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 
 			}
 			long_link_buffer = malloc(TARRECORDSIZE + 1);
+			memset(long_link_buffer, '\0', TARRECORDSIZE + 1);
 			SWLIB_ALLOC_ASSERT(long_link_buffer != NULL);
 
 			if (taru_tape_buffered_read(in_des,
@@ -686,8 +737,7 @@ taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 			if (do_record_header) {
 				strob_set_memlength(taru->headerM,
 						taru->header_lengthM + 512);
-				memcpy(
-					(void*)(strob_str(taru->headerM) +
+				memcpy((void*)(strob_str(taru->headerM) +
 							 taru->header_lengthM),
 					(void*)(long_link_buffer), 512);
 				taru->header_lengthM += 512;
@@ -793,7 +843,7 @@ taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 			taru_otoul(tar_hdr->gid, &file_hdr->c_gid);
 		}
 
-		taru_otoul(tar_hdr->size, &file_hdr->c_filesize);
+		taru_otoumax(tar_hdr->size, &file_hdr->c_filesize);
 		taru_otoul(tar_hdr->mtime, &file_hdr->c_mtime);
 		taru_otoul(tar_hdr->devmajor,
 				(unsigned long *) &file_hdr->c_rdev_maj);
@@ -1317,6 +1367,21 @@ taru_otoul (char * s, unsigned long *n)
   return *s == '\0';
 }
 
+int
+taru_otoumax (char * s, uintmax_t *n)
+{
+  uintmax_t val = 0;
+
+  while (*s == ' ')
+    ++s;
+  while (*s >= '0' && *s <= '7')
+    val = 8 * val + *s++ - '0';
+  while (*s == ' ')
+    ++s;
+  *n = val;
+  return *s == '\0';
+}
+
 int taru_tarheader_check ( char * buffer ) {
   unsigned long sumis, sum;
 
@@ -1325,7 +1390,9 @@ int taru_tarheader_check ( char * buffer ) {
   }
   sumis = (unsigned long) taru_from_oct ( 8 , buffer + 148);
   sum = taru_tar_checksum ((void *) buffer);
-  if ( sum != sumis ) return -1;
+  if ( sum != sumis ) {
+	return -1;
+  }
   return 0;
 }
 
@@ -1352,7 +1419,7 @@ unsigned long taru_tar_checksum (void * hdr)
 }
 
 int
-taru_tape_skip_padding (int in_file_des, int offset, 
+taru_tape_skip_padding (int in_file_des, uintmax_t offset, 
 			enum archive_format archive_format_in)
 {
   int pad;
