@@ -142,13 +142,14 @@ taru_digs_create(void)
 {
 	FILE_DIGS * digs = malloc(sizeof(FILE_DIGS));
 	if (!digs) return digs;
-	taru_digs_init(digs, DIGS_ENABLE_OFF);
+	taru_digs_init(digs, DIGS_ENABLE_OFF, 0);
 	return digs;
 }
 
 void
-taru_digs_init(FILE_DIGS * digs, int enable)
+taru_digs_init(FILE_DIGS * digs, int enable, int poison)
 {
+	digs->do_poisonM = poison;
 	digs->md5[0] = '\0';
 	digs->sha1[0] = '\0';
 	digs->sha512[0] = '\0';
@@ -167,7 +168,7 @@ taru_init_header_digs(struct new_cpio_header * file_hdr)
 	if (file_hdr->digsM == NULL)
 		file_hdr->digsM = taru_digs_create();
 	else
-		taru_digs_init(file_hdr->digsM, DIGS_ENABLE_OFF);
+		taru_digs_init(file_hdr->digsM, DIGS_ENABLE_OFF, 0);
 }
 
 void
@@ -297,6 +298,7 @@ void
 taru_write_preview_line(TARU * taru, struct new_cpio_header *file_hdr)
 {
 	int fd;
+	int flag;
 	int level;
 	char * filename;
 	STROB * buffer;
@@ -313,18 +315,30 @@ taru_write_preview_line(TARU * taru, struct new_cpio_header *file_hdr)
 	buffer = taru->preview_bufferM;
 	filename = ahsStaticGetTarFilename(file_hdr);
 
+	if (isatty(fd)) {
+		flag = ls_list_get_encoding_flag();
+		ls_list_set_encoding_by_lang();
+	}
+
+
 	if (level >= TARU_PV_3) {
 		taru_print_tar_ls_list(buffer, file_hdr, LS_LIST_VERBOSE_L2);
 	} else if (level >= TARU_PV_2) {
 		taru_print_tar_ls_list(buffer, file_hdr, LS_LIST_VERBOSE_L1);
 	} else if (level >= TARU_PV_1) {
-		strob_sprintf(buffer, 0, "%s\n", filename);
+		/* strob_sprintf(buffer, 0, "%s\n", filename); */
+		strob_strcpy(buffer, "");
+		ls_list_safe_print_to_strob(filename, buffer);
+		strob_strcat(buffer, "\n");
 	} else {
 		strob_strcpy(buffer, "");
 	}
 	if (strob_strlen(buffer)) {
 		uxfio_write(fd, strob_str(buffer), strob_strlen(buffer));
 	}
+	if (isatty(fd)) {
+		ls_list_set_encoding_flag(flag);
+	}
 }
 
 int
