@@ -27,7 +27,7 @@ taru_format_translate (int ifd, int ofd, enum archive_format output_format ) {
 	_exit (taru_process_copy_in (ifd, fpipe[1])) ;
    } else if (parent > 0) {
         close (fpipe[1]); 
-        ret = taru_process_copy_out (taru, fpipe [0] , ofd, defer, NULL, output_format ); 
+        ret = taru_process_copy_out (taru, fpipe [0] , ofd, defer, NULL, output_format, -1, -1, (intmax_t*)NULL, NULL); 
         while ( !waitpid ( (pid_t)parent, &status, WNOHANG)) {
                ;
 	}
