@@ -80,7 +80,7 @@ main(int argc, char **argv)
 	//
 	// Construct the export object.
 	//
-	swexdist = swExCat::constructSwExDist(swpsf, &error_code);
+	swexdist = swExCat::constructSwExDist(swpsf, &error_code, NULL);
 
 	//
 	// Write out something that looks like a PSF.
