@@ -31,7 +31,7 @@ main (void)
 
   swspsf=new swsPSF(psf);
   swspsf->generateStructuresFromParser();
-  len=swspsf->write(STDOUT_FILENO);
+  len=swspsf->iwrite(STDOUT_FILENO);
   cerr << len << endl;
 
 }
