@@ -29,6 +29,6 @@ main (void)
   swspsf = new swsPSF(psf);
   swspsf->generateStructures();
   //len=swspsf->write(STDOUT_FILENO);
-  cerr << swspsf->write(STDOUT_FILENO) << endl;
+  cerr << swspsf->iwrite(STDOUT_FILENO) << endl;
 }
 
